# -*- coding: utf-8 -*-

"""Top-level package for Python virtual machine detection library."""

__author__ = """Andres Kepler"""
__email__ = 'andres@kepler.ee'
__version__ = '0.2.1'
from .py_vmdetect import VMDetect  # noqa: F401
